#ifndef ESTIMATORS
#define ESTIMATORS

#include <stdlib.h>
#include <string.h>
#include<stdio.h>
#include<math.h>
#include<sys/types.h>		
#include<iostream.h>
#include<fstream.h>

#include "interface_classes.h"

class Signal{
 public:
  Signal();
  virtual int Index()=0;
  virtual ~Signal();

  int N; 

 protected:
  int Inputs;
 

};

class ContinuousSignal : public Signal{
  
 public:
  ContinuousSignal(int in, double* left, double* right, int* nn);
  void setValue(double* v);
  int Index();
  ~ContinuousSignal();

 

 protected:
  double* LeftBounds;
  double* RightBounds;
  int* n;
  int* IndCoef;
  double* value;


};



///////////////////////////////////////////////////

class EntropyEstimator{

 public:

  EntropyEstimator(int type, int n, double volume=0, double l=1);
  void update(Signal* sig);
  int getCount(Signal* sig);
  double getProbability(Signal* sig);
  bool getProbabilityVector(double* p);
  double getEntropy();
  int getNumberOfSamples();
  ~EntropyEstimator();


 protected:

  int N;
  int Type;
  double Volume;
  double Samples;
  double entropy;
  int* counts;
  double lambda;
  double powerLambda;

  double g(double p);
  double log2(double x);


};


////////////////////////////////////////////////////

class EntropyEstimatorCollection{

 public:

  EntropyEstimatorCollection(int EstimatorsNumber, int type, int EstimatorBins, double volume=0, double l=1 );
  void update(Signal* item, Signal* sample);
  int getCount(Signal* item, Signal* fromBin);
  double getProbability(Signal* item, Signal* fromBin);
  bool getProbabilityVector(Signal* item, double* p);
  double getEntropy(Signal* item);
  int getNumberOfSamples(Signal* item);
  void print();
  ~EntropyEstimatorCollection();

 protected:
  int N;
  EntropyEstimator** items;

};

//////////////////////////////////////

class MeanEstimatorCollection{

 public:
  
  MeanEstimatorCollection(int EstimatorsNumber);
  void update(Signal* item, double sample);
  int getNumberOfSamples(Signal* item);
  double getMean(Signal* item);
  ~MeanEstimatorCollection();

 protected:
  int N;
  double* means;
  int* Samples;

};
//////////////////////////////////////////

class ForgettingMeanSTDEstimatorCollection : public MeanEstimatorCollection{

 public:
  
  ForgettingMeanSTDEstimatorCollection(int EstimatorsNumber, double l,double STDmax);
  void update(Signal* item, double sample);
  void get(Signal* item, double& m, double& d);
  ~ForgettingMeanSTDEstimatorCollection();
  

 protected:
  double lambda;
  double lambdaPower;
  double* STDs;

};
//////////////////////////////////////////





class stdEstimatorCollection{

 public:
  stdEstimatorCollection(int EstimatorsNumber);
  void update(Signal* item, double sample);
  int getNumberOfSamples(Signal* item);
  double getSTD(Signal* item);
  ~stdEstimatorCollection();


 protected:
  int N;
  double* squares;
  double* means;
  int* Samples;

};



////////////////////////////////////////////



class StorageTable{

 public:

  StorageTable(int items);
  void storeItem(Signal* item, double value);
  double getItem(Signal* item);
  double getMean();
  double getSTD();
  void saveTable(char* fileName);
  void loadTable(char* fileName);
  ~StorageTable();

 protected:
  int Items;
  double* values;

};


//--------------------------------------------------



#endif
