/*	This is the implementation of the main() function for
	the program that implements randomized Mountain-Car task, 
	SARSA(lambda) agent with replacing eligibility traces and
	SDMs architecture to represent action value functions.

  Author: Bohdana Ratitch
  Version: November 2003
  file: mcrand_sarsart_sdm.cpp

 */

#include <iostream.h>
#include<time.h>
#include<math.h>
#include<sys/types.h>
#include<stdlib.h>
#include <fstream.h>

#include "interface_classes.h"
#include "mountain_car_rand.h"
#include "sdm.h"
#include "sarsart_agent.h"
#include "main_init.h"

void run(MainParameters& mainP,  StateActionFA* safa, Agent* agent, bool saveFA=true);
double triangularSimilarity(const State& s1, const State& s2, double* threshold);

int main(int argc, char* argv[]){

  int i, d, j;
  char c, *buffer;
	
  int p=0;
  int steps=0;
 

  State::dimensionality=2;

  MainParameters mainP;
  sdmParameters sdmP;

  if (strncmp(argv[1],"?",1)==0){
    cout << "Usage:" << endl;
    cout << "executable_name parameter1=value1 parameter2=value2 ..." << endl;
    SarsaAgentRT::helpLearningParameters();
    cout << "Mountain-Car parameters:" << endl;
    cout << "AV=value : action variance" << endl;
    cout << "PV=value : position variance" << endl;
    p+=mainP.process(argc,argv);
    sdmP.processCommandLine(argc,argv);
    exit(EXIT_SUCCESS);
  }

  p+=mainP.process(argc,argv);
  sdmP.processCommandLine(argc,argv);
  
 
  double ActVar=0, PosVar=0;
  for (i=0; i<argc; i++){
    
    if (strncmp("AV=", argv[i], 3)==0){
      ActVar=atof(&(argv[i][3]));
    }

    if (strncmp("PV=", argv[i], 3)==0){
      PosVar=atof(&(argv[i][3]));
    }
  }
 
  
 //seed randon number generator
  time_t stime;
  time(&stime);
  struct tm* currentTime;
  currentTime=gmtime(&stime);
  unsigned seed;
  seed=(currentTime->tm_hour+1)*currentTime->tm_min*currentTime->tm_sec;
  srand(seed);
 
  //state space and actions for the mountain car task
	
  

  ActionSet as(3);
  Action a1("forward", 1.0);
  as.addAction(a1);
  Action a2("reverse", -1.0);
  as.addAction(a2);
  Action a3("zero", 0.0);
  as.addAction(a3);


  MountainCarRand* mdp = new MountainCarRand(ActVar, PosVar);
  Environment* env = mdp;

	
  //Set parameters for CMAC and 
  //create approximators for each action with appropriate parameters
  
  Approximator** sdmSet = new Approximator*[Action::count];

  

  env->getStateSpaceBounds(sdmP.left, sdmP.right);
  

  for(i=0; i<Action::count; i++){
    sdmSet[i] = new SDM(sdmP, &triangularSimilarity);
    sdmSet[i]->setLearningParameters(argc, argv);
  }

  StateActionFA* safa = new StateActionFA(Action::count, sdmSet);
  

  
  
  //create a learning agent object
  SarsaAgentRT* sarsa = new SarsaAgentRT(1, as, safa, env);
  Agent* agent = sarsa;
  agent->setLearningParameters(argc, argv);

	
  run(mainP, safa, agent, true);
  

  delete sarsa;
  delete safa;
  delete mdp;
 
  return 0;
}
