#include <stdlib.h>
#include <string.h>
#include<stdio.h>
#include<math.h>
#include<sys/types.h>		
#include<iostream.h>
#include<fstream.h>
#include<float.h>
#include<time.h>

#ifndef TARGET_FUNCTION
#define TARGET_FUNCTION

#include "interface_classes.h"

class Target {
	/*	
		Class for managing data samples of the target function
		This is an abstract class. 
	*/

public:

	Target(){	//
	
		DataSetSize = 0;
		TrainingSetSize = 0;
		TestSetSize = 0;
		DataSet = NULL;
		TrainingSet = NULL;
		TestSet = NULL;
	}
	
	virtual void loadDataSet(int argc, char* argv[])=0;
	/*	This function should be implemented in an inherited class. It can use a known 
		(artificial) function to create a data set or load data from some text file.
	*/

	virtual void formTrainingAndTestSets(int trainingSetSize, int testSetSize)=0;
	/*	This function should be implemented for each inherited class. It devides
		data set into training and test samples of specified sizes. 
	*/
	
	void getTrainingSample(State& input, double& target, int sampleNumber=-1)
	/*	The last parameter is optional. Its default value (-1) is used 
		when the user does not provide another value, in which case 
		a random sample is selected and returned.
	*/

	{	if (sampleNumber==-1)
			sampleNumber = rand() % TrainingSetSize;
		
		input = TrainingSet[sampleNumber].input;
		target = TrainingSet[sampleNumber].target;
	}

	void getTestSample(State& input, double& target, int sampleNumber=-1)
	/*	The last parameter is optional. Its default value (-1) is used 
		when the user doues not provide another value, in which case 
		a random sample is selected and returned.
	*/

	{

		if (sampleNumber==-1)
			sampleNumber = rand() % TestSetSize;
		
		input = TestSet[sampleNumber].input;
		target = TestSet[sampleNumber].target;

	}
	
	void shuffleTrainingSamples()
	/* Randomly shuffles samples in the training set
	*/
	
	{
		int ind1, ind2, i;
		Sample s;

		ind1= rand() % TrainingSetSize;
		s = TrainingSet[ind1];
		TrainingSet[ind1].taken=true;

		for (i=0; i<(TrainingSetSize-1); i++){
			ind2=rand() % TrainingSetSize;
			if (TrainingSet[ind2].taken == true ){
				do {
					if (ind2 < (TrainingSetSize-1)) ind2++;
					else ind2=0;
				}
				while (TrainingSet[ind2].taken == true);
			}

			TrainingSet[ind2].taken = true;
			TrainingSet[ind1]=TrainingSet[ind2];
			ind1=ind2;
		}

		TrainingSet[ind2]=s;

		for (i=0; i<TrainingSetSize; i++)
			TrainingSet[i].taken = false;


		
	}


	
	int dataSetSize(){
		return DataSetSize;
	}

	~Target(){
		delete[] DataSet;
		delete[] TrainingSet;
		delete[] TestSet;
	}


protected: 

	int DataSetSize;
	int TrainingSetSize;
	int TestSetSize;

	struct Sample {
		State input;
		double target;
		bool taken;
	};
	
	Sample* DataSet;
	Sample* TrainingSet;
	Sample* TestSet;



};

#endif
